/*
   I, the copyright holder of this work, hereby release it into the public
   domain. This applies worldwide. In case this is not legally possible: I
   grant anyone the right to use this work for any purpose, without any
   conditions, unless such conditions are required by law.
*/

#include "fernandotel.h"

void center_string(struct ncdirect *nc, char *str) {
    ncdirect_cursor_right(nc, ncdirect_dim_x(nc)/2 - (int)ceil(strlen(str)/2));
    printf("%s", str);
}

void display_room(struct ncdirect *nc, struct Player *player, struct Item *items, struct Player *players) {
    unsigned short i, num_things;

    ncdirect_set_fg_rgb(nc, WHITE);
    printf("\nYou are %s at area #%d\n\n", player->room->description, player->room->id);

    if (!(player->bits&PLAYER_NPC)) {
        num_things = 0;
        ncdirect_set_fg_rgb(nc, DARK_RED);
        printf("Here you find: ");
        num_things += display_items_in_room(nc, player->room, items);
        num_things += display_players_in_room(nc, player->room, players, player);
        if (!num_things)
            printf("Nothing !");
        printf("\n");
    }

    printf("\n");

    ncdirect_set_fg_rgb(nc, (player->bits&PLAYER_NPC) ? BRIGHT_CYAN : DARK_GREEN);
    printf("Exits are: ");
    if (player->room->exits[0]->id)
        printf("N)orth, ");
    if (player->room->exits[1]->id)
        printf("S)outh, ");
    if (player->room->exits[3]->id)
        printf("E)ast, ");
    if (player->room->exits[2]->id)
        printf("W)est, ");
    if (player->room->exits[4]->id)
        printf("U)p, ");
    if (player->room->exits[5]->id)
        printf("D)own, ");

    if (player->bits&PLAYER_NPC) {
        printf("C)reate, M)odify, Q)uit ? ");
        fflush(stdout);
        return;
    }

    ncdirect_cursor_left(nc, 2);
    printf("  \n\n");

    ncdirect_set_fg_rgb(nc, BRIGHT_CYAN);
    printf("(%d moves left) Your Command or (H)elp ? ", player->moves);
    fflush(stdout);
}

bool navigate(struct ncdirect *nc, struct Player *player, struct Item *items, struct Player *players) {
    uint32_t key;
    ncinput ni;
    unsigned short idpos;

    if (rand() % 100 < 10) {
        random_encounter(nc, player);
        return true;
    }

    display_room(nc, player, items, players);

    while (1) {
        idpos = 0;
        key = ncdirect_get_blocking(nc, &ni);
        if (key == (uint32_t)-1) {
            printf("ERROR in receiving keypress, bailing!\n");
            return NULL;
        }
        if (ni.evtype && ni.evtype != NCTYPE_PRESS)
            continue;
        switch (key) {
            case 'q':
            case 'Q':
                printf("Q\n");
                return false;
            case 'h':
            case 'H':
            case '?':
                printf("H\n");
                display_navigation_help(nc);
                return true;
            case 't':
            case 'T':
                printf("T\n");
                take_item(nc, player, items);
                return true;
            case 'l':
            case 'L':
                printf("L\n");
                leave_item(nc, player, items);
                return true;
            case 'i':
            case 'I':
                printf("I\n");
                display_inventory(nc, player, items);
                return true;
            case 'k':
            case 'K':
                printf("K\n");
                if (kill_player(nc, player, items) && count_players(players, PLAYER_ALIVE) <= 1) {
                    ncdirect_set_fg_rgb(nc, WHITE);
                    printf("Congratulations, you're the sole survivor!\n");
                }
                return true;
            case 'c':
            case 'C':
                if (!(player->bits&PLAYER_NPC))
                    break;
                create_exit(nc, player->room);
                return true;
            case 'm':
            case 'M':
                if (!(player->bits&PLAYER_NPC))
                    break;
                modify_description(nc, player->room);
                return true;
            case 'd':
            case 'D':
                idpos++;
            case 'u':
            case 'U':
                idpos++;
            case 'e':
            case 'E':
                idpos++;
            case 'w':
            case 'W':
                idpos++;
            case 's':
            case 'S':
                idpos++;
            case 'n':
            case 'N':
                if (key >= 100)
                    key -= 32;
                if (player->room->exits[idpos]->id) {
                    printf("%c\n", key);
                    if (!(player->bits&PLAYER_NPC)) {
                        if (!player_moves_check(nc, player))
                            return true;
                        player->moves--;
                    }
                    player->room = player->room->exits[idpos];
                    return true;
                }
                break;
        }
    }
}

unsigned short display_items_in_room(struct ncdirect *nc, struct Room *room, struct Item *items) {
    unsigned short i;
    unsigned short ret = 0;

    for (i=1; i<MAX_ITEMS; i++) {
        if (items[i].bits&ITEM_IN_ROOM && items[i].location == room->id) {
            display_item(nc, &items[i]);
            ret++;
        }
    }

    return ret;
}

unsigned short display_players_in_room(struct ncdirect *nc, struct Room *room, struct Player *players, struct Player *player) {
    unsigned short i, j;
    unsigned short ret = 0;
    struct Player *stalker;

    for (i=0; i<MAX_PLAYERS; i++) {
        if (   players[i].bits&PLAYER_INITIALIZED
            && players[i].room
            && players[i].room->id == room->id
            && players[i].id != player->id
        ) {
            if (!(players[i].bits&PLAYER_ALIVE)) {
                ncdirect_set_fg_rgb(nc, DARK_CYAN);
                printf("\n%s's dead body!", players[i].name);
            } else {
                ncdirect_set_fg_rgb(nc, WHITE);
                printf("\n%s who is alive!", players[i].name);
                if (players[i].bits&PLAYER_NPC) {
                    ncdirect_set_fg_rgb(nc, BRIGHT_RED);
                    stalker = targeted_by(player, players);
                    printf("\n%s says %s is stalking you!",
                        players[i].name,
                        stalker ? stalker->name : "no one"
                    );
                }
            }
            ret++;
        }
    }

    return ret;
}

void display_navigation_help(struct ncdirect *nc) {
    ncdirect_set_fg_rgb(nc, BROWN);
    printf("(N)orth to move one area north\n");
    printf("(S)outh to move one area south\n");
    printf("(W)est to move one area west\n");
    printf("(E)ast to move one area east\n");
    printf("(U)p to move or climb one area up\n");
    printf("(D)own to move of climb one area down\n");
    printf("(T)ake to take or carry items\n");
    printf("(L)eave to drop or discard items\n");
    printf("(K)ill to attempt killing your target\n");
    printf("(I)nventory to see what you have\n");
    printf("(Q)uit to return to main menu\n");
}

void take_item(struct ncdirect *nc, struct Player *player, struct Item *items) {
    char *str;
    unsigned short item_id, i;

    printf("\n\n");

    if (!player_moves_check(nc, player))
        return;

    if (items_in_player_inventory(player, items) >= CONFIG_MAX_INVENTORY) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("You're carrying too much shit!\n");
        return;
    }

    if (!items_in_room(player->room, items)) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("Floor's empty.  There's nothing here.\n");
        return;
    }

    ncdirect_set_fg_rgb(nc, BRIGHT_CYAN);
    str = ncdirect_readline(nc, "Take which object (number) ? ");
    item_id = (unsigned short)strtoul(str, NULL, 10);
    free(str);

    if (!item_id || item_id > MAX_ITEMS || !items[item_id].id) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("Nice try, wise guy.\n");
        return;
    }

    if (items[item_id].bits&ITEM_IN_ROOM && items[item_id].location == player->room->id) {
        items[item_id].bits &= ~ITEM_IN_ROOM;
        items[item_id].bits |= ITEM_IN_INVENTORY;
        items[item_id].location = player->id;
        player->moves--;
        return;
    }

    ncdirect_set_fg_rgb(nc, BRIGHT_RED);
    printf("That item's not there.\n");
}

void leave_item(struct ncdirect *nc, struct Player *player, struct Item *items) {
    char *str;
    unsigned short item_id, i;

    printf("\n\n");

    if (!player_moves_check(nc, player))
        return;

    if (!display_inventory(nc, player, items))
        return;

    ncdirect_set_fg_rgb(nc, WHITE);
    str = ncdirect_readline(nc, "Leave which object (number) ? ");
    item_id = (unsigned short)strtoul(str, NULL, 10);
    free(str);

    if (!item_id || item_id > MAX_ITEMS || !items[item_id].id) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("Nice try, wise guy.\n");
        return;
    }

    if (items[item_id].bits&ITEM_IN_INVENTORY && items[item_id].location == player->id) {
        items[item_id].bits &= ~ITEM_IN_INVENTORY;
        items[item_id].bits |= ITEM_IN_ROOM;
        items[item_id].location = player->room->id;
        ncdirect_set_fg_rgb(nc, BRIGHT_CYAN);
        printf("Item dropped.\n");
        player->moves--;
    }

    ncdirect_set_fg_rgb(nc, BRIGHT_RED);
    printf("That item's not in your inventory.\n");
}

bool display_inventory(struct ncdirect *nc, struct Player *player, struct Item *items) {
    unsigned short i;

    printf("\n");

    if (items_in_player_inventory(player, items) < 1) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("You're not carrying anything!\n");
        return false;
    }

    ncdirect_set_fg_rgb(nc, BRIGHT_RED);
    printf("You have:");
    for (i=1; i<MAX_ITEMS; i++) {
        if (items[i].bits&ITEM_IN_INVENTORY && items[i].location == player->id)
            display_item(nc, &items[i]);
    }
    if (player->target) {
        ncdirect_set_fg_rgb(nc, BRIGHT_CYAN);
        printf("\nAn urge to kill %s", player->target->name);
    }

    printf("\n");

    return true;
}

void display_item(struct ncdirect *nc, struct Item *item) {
    ncdirect_set_fg_rgb(nc, BRIGHT_CYAN);
    printf("\n# %d ", item->id);
    ncdirect_set_fg_rgb(nc, item->color);
    printf("%s", item->name);
}

bool kill_player(struct ncdirect *nc, struct Player *player, struct Item *items) {
    char str[256];
    char *weapon_pair;
    unsigned short i, c;
    struct Item *part_a;
    struct Item *part_b;

    if (player->target == NULL) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("\nYou have no target selected! Go back to the main menu and pick one.\n");
        return false;
    }

    if (player->room->id != player->target->room->id) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("\nYour target (%s) isn't here!\n", player->target->name);
        return false;
    }

    for (i=ITEM_SET_1; i<=ITEM_SET_9; i=i*2) {
        weapon_pair = find_item_pair_on_player(player, items, i);
        if (!weapon_pair)
            continue;
        ncdirect_set_fg_rgb(nc, BRIGHT_YELLOW);
        sprintf(str, "Attempt to kill %s with the %s", player->target->name, weapon_pair);
        if (!yes_no(nc, str))
            continue;
        sprintf(str, "used the %s to do the job!", weapon_pair);
        free(weapon_pair);
        if (find_item_on_player(player->target, items, i | ITEM_DEFENSE | ITEM_IN_INVENTORY)) {
            ncdirect_set_fg_rgb(nc, BRIGHT_RED);
            printf("Fuck! Your target had the defense for that weapon!\n");
            c = defender_has_weapon_pair(nc, player->target, items);
            if (c && find_item_on_player(player, items, c | ITEM_DEFENSE | ITEM_IN_INVENTORY)) {
                printf("You managed to defend against the counter-attack!\n");
                printf("In a panic you flee to a random area and lose ten moves.\n");
                player->room = NULL;
                player->starting_room = 0;
                if (player->moves <= 10) {
                    player->moves = 0;
                    return false;
                }
                player->moves -= 10;
                return false;
            } else if (c) {
                printf("YOU DIED. So much for that career as a murderer, eh?\n");
                printf("Since you suck so much at this game, you lose all moves for the day.\n");
                printf("Better luck tomorrow!\n");
                player->moves = 0;
                player->bits &= ~PLAYER_ALIVE;
                player->killed++;
                player->target->kills++;
                strcpy(str, "used an undisclosed weapon in self-defense!");
                write_obit(player->target, player, str, player->room);
                return false;
            }
            ncdirect_set_fg_rgb(nc, BRIGHT_YELLOW);
            printf("Luckily for you your target didn't have a complete weapon with which\n");
            printf("to fight back.  Instead, you'll be sent to a random area as a further\n");
            printf("effort to protect the innocent from lunatics like yourself!\n");
            player->room = NULL;
            player->starting_room = 0;
            return false;
        }
        ncdirect_set_fg_rgb(nc, BRIGHT_GREEN);
        printf("Congratulations, you've killed your target!\n\n");
        player->target->bits &= ~PLAYER_ALIVE;
        player->target->killed++;
        write_obit(player, player->target, str, player->room);
        player->target = NULL;
        player->kills++;
        return true;
    }

    ncdirect_set_fg_rgb(nc, BRIGHT_RED);
    printf("\nWhat're you supposed to use, man? Harsh language?\n");
    return false;
}

unsigned short defender_has_weapon_pair(struct ncdirect *nc, struct Player *player, struct Item *items) {
    unsigned short i;
    char *weapon_pair;
    struct Item *part_a;
    struct Item *part_b;

    for (i=ITEM_SET_1; i<=ITEM_SET_9; i=i*2) {
        weapon_pair = find_item_pair_on_player(player, items, i);
        if (!weapon_pair)
            continue;
        printf("%s FIGHTS BACK with the %s\n", player->name, weapon_pair);
        free(weapon_pair);
        return i;
    }

    return 0;
}

void most_wanted(struct ncdirect *nc, struct Player *players) {
    unsigned short i, pad, j;
    struct tm *timeinfo;
    char ymd[11];

    fernando_center(nc, "Fernandotel Guest List");

    printf("\n\n");

    for (i=0; i<MAX_PLAYERS; i++) {
        if (!(players[i].bits&PLAYER_INITIALIZED))
            continue;
        ncdirect_set_fg_rgb(nc, WHITE);
        pad = MAX_PLAYER_NAME-strlen(players[i].name);
        printf("%s", players[i].name);
        for (j=0; j<pad; j++)
            printf(".");
        ncdirect_set_fg_rgb(nc, BRIGHT_GREEN);
        if (players[i].bits&PLAYER_NPC) {
            printf("...(NPC)...");
        } else {
            timeinfo = gmtime(&players[i].moves_t);
            strftime(ymd, 11, "%Y-%m-%d", timeinfo);
            printf("%s.", ymd);
        }
        ncdirect_set_fg_rgb(nc, BRIGHT_CYAN);
        if (players[i].bits&PLAYER_ALIVE)
            printf("Alive..");
        else
            printf("Dead!..");
        ncdirect_set_fg_rgb(nc, BRIGHT_YELLOW);
        printf("Killed..");
        if (players[i].kills < 10)
            printf(".");
        printf("%d..", players[i].kills);
        ncdirect_set_fg_rgb(nc, BRIGHT_CYAN);
        printf("Was Killed..");
        if (players[i].killed < 10)
            printf(".");
        printf("%d..", players[i].killed);
        printf("\n");
    }

    enter_to_continue(nc, GREY);
}

void display_obits(struct ncdirect *nc) {
    FILE *file = fopen("obits.csv", "r");
    char **csv_elements;
    char line[256];
    char ymd[11];
    time_t t;
    struct tm *timeinfo;
    unsigned short i = 0;
    unsigned colors[] = { BRIGHT_CYAN, BRIGHT_GREEN, BRIGHT_RED };

    if (file == NULL) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("\nNobody's died yet!  Why don't you go change that?\n");
        return;
    }

    fernando_center(nc, "Fernandotel Obituaries");

    printf("\n");

    while (fgets(line, 255, file) != NULL) {
        if (i>(sizeof(colors)/sizeof(unsigned)))
            i=0;
        ncdirect_set_fg_rgb(nc, colors[i++]);
        csv_elements = csv_line_to_null_terminated_multiple(line);
        t = (time_t)strtoul(csv_elements[4], NULL, 10);
        timeinfo = gmtime(&t);
        strftime(ymd, 11, "%Y-%m-%d", timeinfo);
        printf("\n");
        printf("%s killed %s on %s\n", csv_elements[0], csv_elements[1], ymd);
        printf("%s %s\n", csv_elements[0], csv_elements[2]);
        printf("The authorities are %s looking for evidence.\n", csv_elements[3]);
        free(csv_elements);
    }

    fclose(file);

    enter_to_continue(nc, GREY);
}

bool squeals_menu(struct ncdirect *nc, struct Player *player, struct Player *players) {
    uint32_t key;
    ncinput ni;
    char str[80];

    ncdirect_set_fg_rgb(nc, DARK_CYAN);
    sprintf(str, "You have %d moves left.\n", player->moves);
    center_string(nc, str);
    printf("\n");
    ncdirect_set_fg_rgb(nc, BRIGHT_YELLOW);
    center_string(nc, "(A)dd to the squeals\n");
    center_string(nc, "(R)ead the squeals  \n");
    center_string(nc, "(Q)uit to main menu \n");
    printf("\n");
    ncdirect_set_fg_rgb(nc, BRIGHT_GREEN);
    center_string(nc, "Your command ? ");
    fflush(stdout);

    while (1) {
        key = ncdirect_get_blocking(nc, &ni);
        if (key == (uint32_t)-1) {
            printf("ERROR in receiving keypress, bailing!\n");
            return false;
        }
        if (ni.evtype && ni.evtype != NCTYPE_PRESS)
            continue;
        switch (key) {
            case 'a':
            case 'A':
                printf("A\n\n");
                add_squeal(nc, player, players);
                return false;
            case 'r':
            case 'R':
                printf("R\n\n");
                read_squeals_ui(nc);
                return false;
            case 'q':
            case 'Q':
                printf("Q\n\n");
                return true;
        }
    }
}

void add_squeal(struct ncdirect *nc, struct Player *player, struct Player *players) {
    struct Room *rooms;
    struct Player **players_alive;
    unsigned short i, player_id, room_id;
    char *str;
    struct Squeal new_squeal, *squeals;

    printf("\n");
    players_alive = find_alive_players(players);
    for (i=0; players_alive[i]; i++) {
        printf("%2d: %s\n", i+1, players_alive[i]->name);
    }
    printf("\n");
    ncdirect_set_fg_rgb(nc, WHITE);
    str = ncdirect_readline(nc, "Which user (number) did you see ? ");
    player_id = (unsigned short)strtoul(str, NULL, 10);
    free(str);
    if (!player_id || player_id > i) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("No such player number - ibm error?\n");
    } else {
        player_id--;
        rooms = initialize_rooms_from_map("fernandotel.map", false /* empty rooms? */);
        check_player_placement(players, rooms);
        printf("\n");
        ncdirect_set_fg_rgb(nc, WHITE);
        str = ncdirect_readline(nc, "In which area (number) did you see them ? ");
        room_id = (unsigned short)strtoul(str, NULL, 10);
        if (!room_id || room_id > count_rooms(rooms)) {
            ncdirect_set_fg_rgb(nc, BRIGHT_RED);
            printf("If you're going to lie, you could at least use an area that exists.\n");
        } else {
            squeals = read_squeals_file();
            strcpy(new_squeal.name, players_alive[player_id]->name);
            strcpy(new_squeal.description, rooms[room_id].description);
            new_squeal.squeal_t = time(NULL);
            new_squeal.truthful = false;
            if (players_alive[player_id]->room->id == room_id)
                new_squeal.truthful = true;
            if (compare_squeals(&new_squeal, squeals)) {
                ncdirect_set_fg_rgb(nc, BRIGHT_RED);
                printf("Sorry, we already have that squeal thank you!\n");
            } else if (player_id == player->id) {
                if (player->moves < 20) {
                    ncdirect_set_fg_rgb(nc, BRIGHT_RED);
                    printf("You can't afford the 20 moves needed to squeal on yourself!\n");
                } else {
                    ncdirect_set_fg_rgb(nc, BRIGHT_CYAN);
                    printf("Squealing on yourself, aye!  You automatically lose 20 big ones!\n");
                    player->moves -= 20;
                    write_squeal(&new_squeal);
                    write_player_data(players);
                }
            } else {
                if (player->moves < 20) {
                    ncdirect_set_fg_rgb(nc, BRIGHT_RED);
                    printf("You can't afford the 20 moves needed to enter a false squeal!\n");
                } else {
                    if (new_squeal.truthful == false) {
                        ncdirect_set_fg_rgb(nc, BRIGHT_GREEN);
                        printf("Ahh, a false squeal!  Delightful!  You lose 20 moves!\n");
                        player->moves -= 20;
                    } else {
                        ncdirect_set_fg_rgb(nc, WHITE);
                        printf("Honesty is always the best policy!  You receive 20 extra moves!\n");
                        player->moves += 20;
                    }
                    write_squeal(&new_squeal);
                    write_player_data(players);
                }
            }
            free(squeals);
        }
        free(rooms);
    }
    free(players_alive);
}

void read_squeals_ui(struct ncdirect *nc) {
    struct Squeal *squeals = read_squeals_file();
    unsigned short i;
    unsigned short truths = 0;
    char str[80];
    struct tm *timeinfo;
    char ymd[11];

    printf("\n");

    if (squeals == NULL) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("There haven't been any squeals recorded yet!\n");
        return;
    }

    fernando_center(nc, "Fernandotel Squeals");

    printf("\n\n");

    for (i=0; squeals[i].truthful != SQUEAL_END; i++) {
        timeinfo = gmtime(&squeals[i].squeal_t);
        strftime(ymd, 11, "%Y-%m-%d", timeinfo);
        ncdirect_set_fg_rgb(nc, DARK_GREY);
        printf("[");
        ncdirect_set_fg_rgb(nc, WHITE);
        printf("%s", ymd);
        ncdirect_set_fg_rgb(nc, DARK_GREY);
        printf("] ");
        ncdirect_set_fg_rgb(nc, BRIGHT_CYAN);
        printf("%s was seen %s\n", squeals[i].name, squeals[i].description);
        truths += squeals[i].truthful;
    }

    printf("\n");

    sprintf(str, "%d of the preceeding squeals were true!", truths);
    fernando_center(nc, str);

    printf("\n");

    enter_to_continue(nc, DARK_CYAN);
}

bool display_file(struct ncdirect *nc, char *filename) {
    FILE *file = fopen(filename, "r");
    char line[1025];
    unsigned short l = 0;
    unsigned y = ncdirect_dim_y(nc)-10;

    if (file == NULL)
        return false;

    fernando_center(nc, filename);

    printf("\n\n");

    ncdirect_set_fg_rgb(nc, GREY);

    while (fgets(line, 1024, file) != NULL) {
        l += wrap_line(nc, line);
        if (l > y) {
            l = 0;
            enter_to_continue(nc, GREY);
        }
    }

    fclose(file);

    enter_to_continue(nc, GREY);

    return true;
}

void random_encounter(struct ncdirect *nc, struct Player *player) {
    unsigned short rnd = rand() % 100;

    ncdirect_set_fg_rgb(nc, BROWN);

    if (rnd < 10) {
        ncdirect_set_fg_rgb(nc, BRIGHT_GREEN);
        if (yes_no(nc, "You've discovered a secret passage! Take it?")) {
            player->room = NULL;
            player->starting_room = 0;
        }
        return;
    } if (rnd < 20) {
        printf("\nYou daydream about open source software.\n");
        return;
    } if (rnd < 30) {
        printf("\nYou daydream about proprietary software.\n");
        return;
    } if (rnd < 40) {
        printf("\nYou feel anxious for no reason at all.\n");
        return;
    } if (rnd < 50) {
        printf("\nYou feel dizzy...\n");
        return;
    } if (rnd < 60) {
        printf("\nYou thought you saw something move. It was probably your imagination.\n");
        return;
    } if (rnd < 70) {
        printf("\nYou catch the glimpse of a rat from the corner of your eye! ~~,=,^>\n");
        return;
    } if (rnd < 80) {
        printf("\nYou found some old crumbs. Who's cleaning this place anyway?\n");
        return;
    } if (rnd < 90) {
        printf("\nFuck! Cobwebs!\n");
        return;
    } else {
        printf("\nYou briefly see a rat shuffle about. Is it Fernando? ~~,=,^>\n");
        return;
    }
}
