/*
   I, the copyright holder of this work, hereby release it into the public
   domain. This applies worldwide. In case this is not legally possible: I
   grant anyone the right to use this work for any purpose, without any
   conditions, unless such conditions are required by law.
*/

#include "fernandotel.h"

struct Item * initialize_items() {
    unsigned short i;

    struct Item *items = (struct Item *)malloc(sizeof(struct Item)*MAX_ITEMS);

    for (i=1; i<19; i++) {
        items[i].id = i;
        strcpy(items[i].name, "pistol");
        items[i].color = WHITE;
        items[i].bits = ITEM_SET_1 | ITEM_PART_A;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "bullets");
        items[i].color = WHITE;
        items[i].bits = ITEM_SET_1 | ITEM_PART_B;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "flak jacket");
        items[i].color = WHITE;
        items[i].bits = ITEM_SET_1 | ITEM_DEFENSE;
    }

    for (; i<37; i++) {
        items[i].id = i;
        strcpy(items[i].name, "gas sprayer");
        items[i].color = BRIGHT_GREEN;
        items[i].bits = ITEM_SET_2 | ITEM_PART_A;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "chlorine");
        items[i].color = BRIGHT_GREEN;
        items[i].bits = ITEM_SET_2 | ITEM_PART_B;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "gas mask");
        items[i].color = BRIGHT_GREEN;
        items[i].bits = ITEM_SET_2 | ITEM_DEFENSE;
    }

    for (; i<55; i++) {
        items[i].id = i;
        strcpy(items[i].name, "axe handle");
        items[i].color = BRIGHT_PURPLE;
        items[i].bits = ITEM_SET_3 | ITEM_PART_A;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "axe blade");
        items[i].color = BRIGHT_PURPLE;
        items[i].bits = ITEM_SET_3 | ITEM_PART_B;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "riot shield");
        items[i].color = BRIGHT_PURPLE;
        items[i].bits = ITEM_SET_3 | ITEM_DEFENSE;
    }

    for (; i<73; i++) {
        items[i].id = i;
        strcpy(items[i].name, "flame thrower");
        items[i].color = BRIGHT_CYAN;
        items[i].bits = ITEM_SET_4 | ITEM_PART_A;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "fuel");
        items[i].color = BRIGHT_CYAN;
        items[i].bits = ITEM_SET_4 | ITEM_PART_B;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "extinguisher");
        items[i].color = BRIGHT_CYAN;
        items[i].bits = ITEM_SET_4 | ITEM_DEFENSE;
    }

    for (; i<91; i++) {
        items[i].id = i;
        strcpy(items[i].name, "acid sprayer");
        items[i].color = BRIGHT_YELLOW;
        items[i].bits = ITEM_SET_1 | ITEM_PART_A;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "acid");
        items[i].color = BRIGHT_YELLOW;
        items[i].bits = ITEM_SET_1 | ITEM_PART_B;

        i++;
        items[i].id = i;
        strcpy(items[i].name, "lye");
        items[i].color = BRIGHT_YELLOW;
        items[i].bits = ITEM_SET_1 | ITEM_DEFENSE;
    }

    return items;
}

void check_item_placement(struct Item *items, struct Room *rooms) {
    unsigned short num_rooms = count_rooms(rooms);
    unsigned short i, roomid;

    for (i=1; i<MAX_ITEMS; i++) {
        if (!(items[i].bits&ITEM_IN_ROOM) && !(items[i].bits&ITEM_IN_INVENTORY)) {
            roomid = rand() % num_rooms + 1;
            while (!rooms[roomid].id)
                roomid = rand() % num_rooms + 1;
            items[i].location = rooms[roomid].id;
            items[i].bits |= ITEM_IN_ROOM;
        }
    }
}

struct Item * find_item_on_player(struct Player *player, struct Item *items, unsigned short bitfield) {
    unsigned short i;

    for (i=1; i<MAX_ITEMS; i++) {
        if ((items[i].bits&bitfield) == (bitfield) && items[i].location == player->id)
            return &items[i];
    }

    return NULL;
}

char * find_item_pair_on_player(struct Player *player, struct Item *items, unsigned short bitfield) {
    char str[256], *ret;
    struct Item *part_a, *part_b;

    part_a = find_item_on_player(player, items, bitfield | ITEM_PART_A | ITEM_IN_INVENTORY);
    if (!part_a)
        return NULL;
    part_b = find_item_on_player(player, items, bitfield | ITEM_PART_B | ITEM_IN_INVENTORY);
    if (!part_b)
        return NULL;

    sprintf(str, "%s and %s", part_a->name, part_b->name);
    ret = malloc(strlen(str)+1);
    strcpy(ret, str);

    return ret;
}

unsigned short items_in_room(struct Room *room, struct Item *items) {
    unsigned short ret = 0;
    unsigned short i;

    for (i=1; i<MAX_ITEMS; i++) {
        if (!(items[i].bits&ITEM_IN_INVENTORY) && items[i].location == room->id)
            ret++;
    }

    return ret;
}

bool write_item_data(struct Item *items) {
    char str[25] = "fernando-items-XXXXXX";
    int tmp = mkstemp(str);
    unsigned short i;

    if (tmp == -1)
        return false;

    for (i=1; i<MAX_ITEMS; i++) {
        dprintf(tmp, "\"%s\",%u,%d,%d\r\n",
            items[i].name,
            items[i].color,
            items[i].bits,
            items[i].location
        );
    }

    close(tmp);

    if (rename(str, "items.csv") == -1)
        return false;

    return true;
}

bool read_item_data(struct Item *items) {
    FILE *file = fopen("items.csv", "r");
    char **csv_elements;
    char line[256];
    unsigned short i = 1;
    unsigned short tmp;

    if (file == NULL) {
        write_item_data(items);
        file = fopen("items.csv", "r");
        if (file == NULL) {
            printf("Error opening items.csv - Bailing!\n");
            return false;
        }
    }

    while (fgets(line, 255, file) != NULL) {
        csv_elements = csv_line_to_null_terminated_multiple(line);
        strcpy(items[i].name, csv_elements[0]);
        items[i].color = strtoul(csv_elements[1], NULL, 10);
        items[i].bits = (unsigned short)strtoul(csv_elements[2], NULL, 10);
        items[i].location = (unsigned short)strtoul(csv_elements[3], NULL, 10);
        free(csv_elements);
        i++;
    }

    return true;
}
