/*
   I, the copyright holder of this work, hereby release it into the public
   domain. This applies worldwide. In case this is not legally possible: I
   grant anyone the right to use this work for any purpose, without any
   conditions, unless such conditions are required by law.
*/

#include "fernandotel.h"

unsigned short count_lines_in_file(const char *filename) {
    FILE *file = fopen(filename, "r");
    char line[81];
    unsigned short ret = 0;
    while (fgets(line, 80, file) != NULL)
        ret++;
    fclose(file);
    return ret;
}

struct Room * initialize_rooms_from_map(const char *filename, bool empty_rooms) {
    FILE *file = fopen(filename, "r");
    char line[81];
    char *lineptr;
    unsigned short delimiter, i, num_rooms;
    unsigned short roomid = 0;
    struct Room *rooms;

    if (empty_rooms)
        num_rooms = 1000; /* for testmap to create new rooms */
    else
        num_rooms = count_lines_in_file(filename)+1;

    rooms = (struct Room *)malloc(num_rooms*sizeof(struct Room));

    for (i=0; i<=num_rooms; i++)
        rooms[i].id = 0;

    if (!file) {
        printf("Error opening map: %s\n", filename);
        return NULL;
    }

    /* First line is map description */
    if (fgets(line, 80, file) != NULL) {
        lineptr = line + 1;
        delimiter = strcspn(lineptr, "\"");
        /* read description here? */
    }

    while (fgets(line, 80, file) != NULL) {
        roomid++;

        rooms[roomid].id = roomid;

        lineptr = line;

        line[strcspn(line, "\r")] = '\0';
        lineptr++; /* Skip over first " */

        /* Room Description */
        delimiter = strcspn(lineptr, "\"");
        lineptr[delimiter] = '\0';
        strcpy(rooms[roomid].description, lineptr);
        lineptr += delimiter + 3; /* "," */

        /* Directions */
        delimiter = strcspn(lineptr, "\"");
        lineptr[delimiter] = '\0';
        /* skipping over NSWEUD */
        lineptr += delimiter + 2; /* ", */

        /* Room ID's */
        for (i=0; i<=5; i++) { /* NSWEUD */
            delimiter = strcspn(lineptr, ",");
            lineptr[delimiter] = '\0';
            rooms[roomid].exits[i] = &rooms[(unsigned short)strtoul(lineptr, NULL, 10)];
            lineptr += delimiter + 1; /* , */
        }
    }

    fclose(file);

    return rooms;
}

unsigned short count_rooms(struct Room *rooms) {
    unsigned short ret = 1;

    while (rooms[ret].id != 0)
        ret++;

    return ret;
}

bool create_empty_map(char *filename) {
    FILE *file = fopen(filename, "w");

    if (file == NULL)
        return false;

    fputs("\"New Map\",1\r\n", file);
    fputs("\"in an empty room\",\"\",0,0,0,0,0,0\r\n", file);

    if (fclose(file) == 0)
        return true;

    return false;
}

bool write_map(char *filename, struct Room *rooms) {
    char str[25] = "fernando-map-XXXXXX";
    int tmp = mkstemp(str);
    unsigned short i = 1;

    if (tmp == -1)
        return false;

    dprintf(tmp, "\"%s\",%d\r\n", filename, count_rooms(rooms)-1);

    while (rooms[i].id) {
        dprintf(tmp, "\"%s\",\"%s%s%s%s%s%s\",%d,%d,%d,%d,%d,%d\r\n",
            rooms[i].description,
            rooms[i].exits[0]->id ? "N" : "",
            rooms[i].exits[1]->id ? "S" : "",
            rooms[i].exits[2]->id ? "W" : "",
            rooms[i].exits[3]->id ? "E" : "",
            rooms[i].exits[4]->id ? "U" : "",
            rooms[i].exits[5]->id ? "D" : "",
            rooms[i].exits[0]->id ? rooms[i].exits[0]->id : 0,
            rooms[i].exits[1]->id ? rooms[i].exits[1]->id : 0,
            rooms[i].exits[2]->id ? rooms[i].exits[2]->id : 0,
            rooms[i].exits[3]->id ? rooms[i].exits[3]->id : 0,
            rooms[i].exits[4]->id ? rooms[i].exits[4]->id : 0,
            rooms[i].exits[5]->id ? rooms[i].exits[5]->id : 0
        );
        i++;
    }

    close(tmp);

    if (rename(str, filename) == -1)
        return false;

    return true;
}

void create_exit(struct ncdirect *nc, struct Room *room) {
    uint32_t key;
    ncinput ni;
    unsigned short idpos;
    char *str;

    printf("\nCreate in what direction? ");

    if (!room->exits[0]->id)
        printf("N)orth, ");
    if (!room->exits[1]->id)
        printf("S)outh, ");
    if (!room->exits[3]->id)
        printf("E)ast, ");
    if (!room->exits[2]->id)
        printf("W)est, ");
    if (!room->exits[4]->id)
        printf("U)p, ");
    if (!room->exits[5]->id)
        printf("D)own, ");

    printf("Q)uit ? ");
    fflush(stdout);

    while (1) {
        idpos = 0;
        key = ncdirect_get_blocking(nc, &ni);
        if (key == (uint32_t)-1) {
            printf("ERROR in receiving keypress, bailing!\n");
            return;
        }
        if (ni.evtype && ni.evtype != NCTYPE_PRESS)
            continue;
        switch (key) {
            case 'd':
            case 'D':
                idpos++;
            case 'u':
            case 'U':
                idpos++;
            case 'e':
            case 'E':
                idpos++;
            case 'w':
            case 'W':
                idpos++;
            case 's':
            case 'S':
                idpos++;
            case 'n':
            case 'N':
                if (key >= 100)
                    key -= 32;
                if (!room->exits[idpos]->id) {
                    printf("%c\n", key);
                    room->exits[idpos] = NULL;
                    return;
                }
                break;
            case 'q':
            case 'Q':
                printf("Q\n");
                return;
        }
    }
}

void modify_description(struct ncdirect *nc, struct Room *room) {
    char *str;

    printf("\n");

    str = ncdirect_readline(nc, "You are ");

    strcpy(room->description, str);

    free(str);
}
