/*
   I, the copyright holder of this work, hereby release it into the public
   domain. This applies worldwide. In case this is not legally possible: I
   grant anyone the right to use this work for any purpose, without any
   conditions, unless such conditions are required by law.
*/

#include "fernandotel.h"

struct Player * initialize_players() {
    unsigned short i;

    struct Player *players = (struct Player *)malloc(MAX_PLAYERS*sizeof(struct Player));

    for (i=0; i<MAX_PLAYERS; i++) {
        players[i].id = i;
        players[i].bits = 0;
    }

    strcpy(players[0].name, "The Maid");
    strcpy(players[1].name, "The Butler");
    strcpy(players[2].name, "The Janitor");
    strcpy(players[3].name, "Fernando the Rat ~~,=,^>");

    for (i=0; i<=3; i++)
        players[i].bits = PLAYER_INITIALIZED | PLAYER_ALIVE | PLAYER_NPC;

    return players;
}

void player_start(struct ncdirect *nc, struct Player *player, struct Player *players) {
    struct Room *rooms = initialize_rooms_from_map("fernandotel.map", false /* empty rooms? */);
    struct Item *items = initialize_items();
    struct Room *new_room, *room;
    unsigned short num_rooms = count_rooms(rooms);
    unsigned short i;

    seed_rand();

    read_item_data(items);
    check_item_placement(items, rooms);

    while (1) {
        check_player_placement(players, rooms);
        write_player_data(players);
        write_item_data(items);
        if (!navigate(nc, player, items, players))
            break;
        if (!(player->bits&PLAYER_ALIVE))
            break;
    }

    write_player_data(players);
    write_item_data(items);

    for (i=0; i<MAX_PLAYERS; i++) {
        if (players[i].bits&PLAYER_INITIALIZED) {
            players[i].starting_room = players[i].room->id;
            players[i].room = NULL;
        }
    }

    free(items);
    free(rooms);

    return;
}

bool write_player_data(struct Player *players) {
    char str[25] = "fernando-players-XXXXXX";
    int tmp = mkstemp(str);
    unsigned short i;

    if (tmp == -1)
        return false;

    for (i=0; i<MAX_PLAYERS; i++) {
        if (!(players[i].bits&PLAYER_INITIALIZED)) {
            dprintf(tmp, "\"\",\"*\",0,0,0,0,0,0,0\r\n");
            continue;
        }
        dprintf(tmp, "\"%s\",\"%s\",%d,%d,%d,%d,%ld,%d,%d\r\n",
            players[i].name,
            players[i].password,
            (players[i].room == NULL) ? players[i].starting_room : players[i].room->id,
            (players[i].target == NULL) ? 0 : players[i].target->id,
            players[i].bits,
            players[i].moves,
            players[i].moves_t,
            players[i].kills,
            players[i].killed
        );
    }

    close(tmp);

    if (rename(str, "players.csv") == -1)
        return false;

    return true;
}

bool read_player_data(struct Player *players) {
    FILE *file = fopen("players.csv", "r");
    char **csv_elements;
    char line[256];
    unsigned short i = 0;
    unsigned short tmp;

    if (file == NULL) {
        write_player_data(players);
        file = fopen("players.csv", "r");
        if (file == NULL) {
            printf("Error opening players.csv - Bailing!\n");
            return false;
        }
    }

    while (fgets(line, 255, file) != NULL) {
        csv_elements = csv_line_to_null_terminated_multiple(line);
        strcpy(players[i].name, csv_elements[0]);
        strcpy(players[i].password, csv_elements[1]);
        tmp = (unsigned short)strtoul(csv_elements[2], NULL, 10);
        players[i].starting_room = (tmp) ? tmp : 0;
        tmp = (unsigned short)strtoul(csv_elements[3], NULL, 10);
        if (tmp)
            players[i].target = &players[tmp];
        tmp = (unsigned short)strtoul(csv_elements[4], NULL, 10);
        players[i].bits = (tmp) ? tmp : 0;
        tmp = (unsigned short)strtoul(csv_elements[5], NULL, 10);
        if (tmp)
            players[i].moves = tmp;
        players[i].moves_t = (time_t)strtoul(csv_elements[6], NULL, 10);
        players[i].kills = (unsigned short)strtoul(csv_elements[7], NULL, 10);
        players[i].killed = (unsigned short)strtoul(csv_elements[8], NULL, 10);
        free(csv_elements);
        i++;
    }

    return true;
}

struct Player * match_player_name(char *str, struct Player *players) {
    char ucname[MAX_PLAYER_NAME+1];
    unsigned short i;

    uc_str(str);

    for (i=0; i<MAX_PLAYERS; i++) {
        if (players[i].bits&PLAYER_INITIALIZED && !(players[i].bits&PLAYER_NPC)) {
            strncpy(ucname, players[i].name, MAX_PLAYER_NAME);
            uc_str(ucname);
            if (strcmp(str, ucname) == 0)
                return &players[i];
        }
    }

    return NULL;
}

bool check_player_password(char *str, struct Player *player) {
    if (strcmp("*", player->password) == 0)
        return false;
    if (strcmp(str, player->password) == 0)
        return true;
    return false;
}

struct Player * create_new_player(struct Player *players) {
    struct Player *player;
    unsigned short i;

    for (i=0; i<MAX_PLAYERS; i++) {
        if (!(players[i].bits&PLAYER_INITIALIZED)) {
            player = &players[i];

            player->id = i;

            player->room = NULL;
            player->starting_room = 1;

            player->moves = CONFIG_MOVES_PER_DAY;
            player->moves_t = time(NULL);

            player->bits = PLAYER_INITIALIZED | PLAYER_ALIVE;

            player->target = NULL;

            return player;
        }
    }

    return NULL;
}

unsigned short items_in_player_inventory(struct Player *player, struct Item *items) {
    unsigned short i;
    unsigned short ret = 0;

    for (i=1; i<MAX_ITEMS; i++) {
        if (items[i].bits&ITEM_IN_INVENTORY && items[i].location == player->id)
            ret++;
    }

    return ret;
}

bool player_moves_check(struct ncdirect *nc, struct Player *player) {
    if (player->moves < 1) {
        ncdirect_set_fg_rgb(nc, BRIGHT_RED);
        printf("\nYou don't have any more moves!  You'll receive more moves tomorrow.\n");
        return false;
    }
    return true;
}

unsigned short count_players(struct Player *players, unsigned short bitfield) {
    unsigned short i;
    unsigned short ret = 0;

    for (i=0; i<MAX_PLAYERS; i++) {
        if (players[i].bits&bitfield && !(players[i].bits&PLAYER_NPC))
            ret++;
    }

    return ret;
}

void check_player_placement(struct Player *players, struct Room *rooms) {
    unsigned short num_rooms = count_rooms(rooms);
    unsigned short i;

    for (i=0; i<MAX_PLAYERS; i++) {
        if (players[i].bits&PLAYER_INITIALIZED) {
            while (!players[i].room || !players[i].room->id) {
                if (players[i].starting_room) {
                    players[i].room = &rooms[players[i].starting_room];
                } else {
                    players[i].room = &rooms[rand() % num_rooms+1];
                }
            }
        }
    }
}

bool is_target(struct Player *player, struct Player *players) {
    unsigned short i;

    for (i=0; i<MAX_PLAYERS; i++) {
        if (!players[i].target)
            continue;
        if (players[i].target->id == player->id)
            return true;
    }

    return false;
}

struct Player * targeted_by(struct Player *player, struct Player *players) {
    unsigned short i;

    for (i=0; i<MAX_PLAYERS; i++) {
        if (   players[i].bits&PLAYER_INITIALIZED
            && players[i].target
            && players[i].target->id == player->id
        ) {
            return &players[i];
        }
    }

    return NULL;
}

struct Player ** find_untargeted_players(struct Player *player, struct Player *players) {
    unsigned short i;
    struct Player **ret = (struct Player **)malloc(sizeof(struct Player *)*MAX_PLAYERS);
    unsigned short r = 0;

    for (i=0; i<MAX_PLAYERS; i++) {
        if (   !(players[i].bits&PLAYER_INITIALIZED)
            || players[i].id == player->id
            || !(players[i].bits&PLAYER_ALIVE)
            || is_target(&players[i], players)
        ) {
            continue;
        }
        ret[r++] = &players[i];
    }

    ret[r] = NULL;

    return ret;
}

struct Player ** find_alive_players(struct Player *players) {
    unsigned short i;
    struct Player **ret = (struct Player **)malloc(sizeof(struct Player *)*MAX_PLAYERS);
    unsigned short r = 0;

    for (i=0; i<MAX_PLAYERS; i++) {
        if (!(players[i].bits&PLAYER_INITIALIZED) || !(players[i].bits&PLAYER_ALIVE))
            continue;
        ret[r++] = &players[i];
    }

    ret[r] = NULL;

    return ret;
}

bool write_obit(struct Player *killer, struct Player *victim, char *weaponstr, struct Room *room) {
    FILE *file = fopen("obits.csv", "r");
    char str[25] = "fernando-obit-XXXXXX";
    char line[256];
    int tmp = mkstemp(str);

    if (tmp == -1) {
        fclose(file);
        return false;
    }

    if (file != NULL) {
        while (fgets(line, 255, file) != NULL)
            dprintf(tmp, "%s", line);
        fclose(file);
    }

    dprintf(tmp, "\"%s\",\"%s\",\"%s\",\"%s\",%ld\r\n",
        killer->name,
        victim->name,
        weaponstr,
        room->description,
        time(NULL)
    );

    close(tmp);

    if (rename(str, "obits.csv") == -1)
        return false;

    return true;
}

struct Player * find_player_by_name(struct Player *players, char *str) {
    unsigned short i;

    for (i=1; players[i].id; i++) {
        if (strcmp(players[i].name, str) == 0)
            return &players[i];
    }

    return NULL;
}
