/*
   I, the copyright holder of this work, hereby release it into the public
   domain. This applies worldwide. In case this is not legally possible: I
   grant anyone the right to use this work for any purpose, without any
   conditions, unless such conditions are required by law.
*/

#include "fernandotel.h"

int main(int argc, char *argv[]) {
    unsigned short i, n;
    struct ncdirect *nc;
    struct Room *rooms, *room, *new_room;
    struct Player player;
    char *str;

    if (argc < 2) {
        printf("You must provide a .map file as the first argument.\n");
        return EXIT_FAILURE;
    }

    nc = ncdirect_init(NULL, NULL, 0);

    if (nc == NULL) {
        printf("Couldn't initialize notcurses.\n");
        return EXIT_FAILURE;
    }

    ncdirect_cursor_enable(nc);

    if (access(argv[1], F_OK) == -1) {
        if (!yes_no(nc, "Couldn't open file, create an empty map?"))
            return ncdirect_stop(nc) ? EXIT_FAILURE : EXIT_SUCCESS;
        create_empty_map(argv[1]);
    }

    rooms = initialize_rooms_from_map(argv[1], true /* empty rooms? */);

    player.bits = PLAYER_NPC;
    player.room = &rooms[1];

    while (1) {
        for (i=0; i<=5; i++) { /* NSWEUD */
            if (player.room->exits[i] == NULL) {
                str = ncdirect_readline(nc, "Exit to which room number (N for new) ? ");
                if (str[0] == 'N') {
                    n = count_rooms(rooms);
                    rooms[n].id = n;
                    strcpy(rooms[n].description, "in an empty room");
                    rooms[n].exits[0] = &rooms[0];
                    rooms[n].exits[1] = &rooms[0];
                    rooms[n].exits[2] = &rooms[0];
                    rooms[n].exits[3] = &rooms[0];
                    rooms[n].exits[4] = &rooms[0];
                    rooms[n].exits[5] = &rooms[0];
                    player.room->exits[i] = &rooms[n];
                } else {
                    player.room->exits[i] = &rooms[(unsigned short)strtoul(str, NULL, 10)];
                }
                free(str);
            }
        }
        if (!navigate(nc, &player, NULL, NULL))
            break;
    }

    if (yes_no(nc, "Save changes to map?")) {
        if (!write_map(argv[1], rooms))
            printf("Couldn't write map file - changes not saved.\n");
    }

    return ncdirect_stop(nc) ? EXIT_FAILURE : EXIT_SUCCESS;
}
