/*
   I, the copyright holder of this work, hereby release it into the public
   domain. This applies worldwide. In case this is not legally possible: I
   grant anyone the right to use this work for any purpose, without any
   conditions, unless such conditions are required by law.
*/

#include "fernandotel.h"

void uc_str(char *str) {
    while (*str) {
        *str = toupper((unsigned char)*str);
        str++;
    }
}

char ** csv_line_to_null_terminated_multiple(char *str) {
    char **ret = (char **)malloc(sizeof(char **) * MAX_CSV_ELEMENTS);
    size_t tmp;
    unsigned short i;

    for (i=0; *str != '\0' && *str != '\n'; i++) {
        if (i > MAX_CSV_ELEMENTS) {
            printf("ERROR: Overflow reading data\n");
            ret[0] = NULL;
            return ret;
        }
        if (*str == '"') {
            str++;
            ret[i] = str;
            tmp = 0;
            while (1) {
                tmp += strcspn(str, "\"");
                if (*(str+tmp+1) == ',' || *(str+tmp+1) == '\r') {
                    *(str+tmp) = '\0';
                    break;
                }
            }
            str += tmp + 2;
            continue;
        }
        ret[i] = str;
        tmp = strcspn(str, ",\r");
        if (*(str+tmp) == '\r' || *(str+tmp) == ',')
            *(str+tmp) = '\0';
        str += tmp + 1;
    }

    ret[i] = NULL;

    return ret;
}


bool yes_no(struct ncdirect *nc, char *str) {
    uint32_t key;
    ncinput ni;

    printf("%s [Yes/No]? ", str);
    fflush(stdout);
    while (1) {
        key = ncdirect_get_blocking(nc, &ni);
        if (key == (uint32_t)-1) {
            printf("ERROR in receiving keypress, bailing!\n");
            return NULL;
        }
        if (ni.evtype && ni.evtype != NCTYPE_PRESS)
            continue;
        switch (key) {
            case 'y':
            case 'Y':
                printf("Y\n");
                return true;
            case 'n':
            case 'N':
                printf("N\n");
                return false;
        }
    }
}

void seed_rand() {
    FILE *file = fopen("/dev/urandom", "r");
    unsigned int seed;

    if (file)
        fread(&seed, 1, sizeof(seed), file);

    fclose(file);

    srand(seed);
}

void alphanumeric_only(char *src, char *dst) {
    unsigned int i;
    unsigned int d = 0;

    for (i=0; i<strlen(src); i++) {
        if (src[i] == ' ' || (src[i] <= 'Z' && src[i] >= 'A') || (src[i] <= 'z' && src[i] >= 'a'))
            dst[d++] = src[i];
    }

    dst[d] = '\0';
    return;
}

unsigned short wrap_line(struct ncdirect *nc, char *str) {
    unsigned short c;
    size_t s;
    char *tok;
    unsigned x = ncdirect_dim_x(nc)-1;
    unsigned short ret = 1;

    printf("   ");
    c = 3;

    for (tok=strtok(str," "); tok; tok=strtok(NULL," ")) {
        s = strlen(tok)+1;
        if (s+c > x) {
            ret++;
            c = 0;
            printf("\n");
        } else {
            printf(" ");
        }
        c += s;
        printf("%s", tok);
    }

    return ret;
}

void enter_to_continue(struct ncdirect *nc, unsigned color) {
    char *unused;

    printf("\n");

    fernando_center(nc, "Hit ENTER to Continue");

    unused = ncdirect_readline(nc, ""); /* avoiding compile warnings here */

    printf("\n");
    ncdirect_set_fg_rgb(nc, color);
}

void fernando_center(struct ncdirect *nc, char *str) {
    unsigned short l = strlen(str);

    l += 16; /* ~~,=,^>  <^,=,~~ */

    ncdirect_cursor_right(nc, ncdirect_dim_x(nc)/2 - l/2);

    ncdirect_set_fg_rgb(nc, BROWN);
    printf("~~,=,^> ");

    ncdirect_set_fg_rgb(nc, BRIGHT_PURPLE);
    printf("%s", str);

    ncdirect_set_fg_rgb(nc, BROWN);
    printf(" <^,=,~~");
}

bool check_pid() {
    FILE *file;
    char line[80];
    pid_t filepid;
    pid_t pid = getpid();

    if (access("fernandotel.pid", F_OK) != -1) {
        file = fopen("fernandotel.pid", "r");
        if (file == NULL)
            return false;
        fgets(line, 80, file);
        filepid = (pid_t)strtoul(line, NULL, 10);
        fclose(file);
	if (kill(filepid, 0) == 0) { /* PID exists */
	    if (is_process_alive(filepid))
                return false;
        }
        unlink("fernandotel.pid");
    }

    sprintf(line, "%d", pid);

    file = fopen("fernandotel.pid", "w");
    fputs(line, file);
    fclose(file);
    return true;
}

bool is_process_alive(pid_t pid) {
    int status;
    pid_t result = waitpid(pid, &status, WNOHANG);

    if (!result)
        return true;

    if (result) {
        if (WIFEXITED(status))
	    return false;
        return true;
    }

    /* error */
    return true;
}

